@@ -1,6 +1,6 @@
 /* Branch trace support for GDB, the GNU debugger.
 
-   Copyright (C) 2013-2025 Free Software Foundation, Inc.
+   Copyright (C) 2013-2026 Free Software Foundation, Inc.
 
    Contributed by Intel Corp. <markus.t.metzger@intel.com>
 
@@ -1393,7 +1393,7 @@ handle_pt_insn_events (struct btrace_thread_info *btinfo,
 	      continue;
 
 	    if (!ptw_string.has_value ())
-	      *ptw_string = hex_string (event.variant.ptwrite.payload);
+	      ptw_string = hex_string (event.variant.ptwrite.payload);
 
 	    handle_pt_aux_insn (btinfo, *ptw_string, pc);
 
