@@ -47,6 +47,13 @@ using namespace windows_nat;
 
 static struct x86_debug_reg_state debug_reg_state;
 
+/* The inferior's target description.  This is a global because the
+   Windows ports support neither bi-arch nor multi-process.  */
+static const_target_desc_up win32_tdesc;
+#ifdef __x86_64__
+static const_target_desc_up wow64_win32_tdesc;
+#endif
+
 static void
 update_debug_registers (thread_info *thread)
 {
@@ -209,18 +216,25 @@ x86_stopped_by_watchpoint (void)
   return x86_dr_stopped_by_watchpoint (&debug_reg_state);
 }
 
-static CORE_ADDR
-x86_stopped_data_address (void)
+static std::vector<CORE_ADDR>
+x86_stopped_data_addresses ()
 {
   CORE_ADDR addr;
   if (x86_dr_stopped_data_address (&debug_reg_state, &addr))
-    return addr;
-  return 0;
+    return { addr };
+  return {};
 }
 
 static void
-i386_initial_stuff (void)
+i386_initial_stuff (process_info *proc)
 {
+#ifdef __x86_64__
+  if (windows_process.wow64_process)
+    proc->tdesc = wow64_win32_tdesc.get ();
+  else
+#endif
+    proc->tdesc = win32_tdesc.get ();
+
   x86_low_init_dregs (&debug_reg_state);
 }
 
@@ -618,6 +632,15 @@ i386_win32_set_pc (struct regcache *regcache, CORE_ADDR pc)
     }
 }
 
+/* Implement win32_target_ops "is_sw_breakpoint" method.  */
+
+static bool
+i386_is_sw_breakpoint (const EXCEPTION_RECORD *er)
+{
+  return (er->ExceptionCode == EXCEPTION_BREAKPOINT
+	  || er->ExceptionCode == STATUS_WX86_BREAKPOINT);
+}
+
 struct win32_target_ops the_low_target = {
   i386_arch_setup,
   i386_win32_num_regs,
@@ -637,5 +660,6 @@ struct win32_target_ops the_low_target = {
   i386_insert_point,
   i386_remove_point,
   x86_stopped_by_watchpoint,
-  x86_stopped_data_address
+  x86_stopped_data_addresses,
+  i386_is_sw_breakpoint,
 };
