@@ -1,5 +1,5 @@
 /* gdb_file_up, an RAII wrapper around FILE.
-   Copyright (C) 2021-2022 Free Software Foundation, Inc.
+   Copyright (C) 2021-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,8 +16,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef GDBSUPPORT_GDB_FILE
-#define GDBSUPPORT_GDB_FILE
+#ifndef GDBSUPPORT_GDB_FILE_H
+#define GDBSUPPORT_GDB_FILE_H
 
 #include <memory>
 #include <stdio.h>
@@ -34,4 +34,4 @@ struct gdb_file_deleter
 
 typedef std::unique_ptr<FILE, gdb_file_deleter> gdb_file_up;
 
-#endif
+#endif /* GDBSUPPORT_GDB_FILE_H */
